@@ -23,8 +23,7 @@
 
 /**
  * \ingroup core
- * Evaluates NMEA sentences coming from a GPS device
- * \since QGIS 1.4
+ * \brief Evaluates NMEA sentences coming from a GPS device
  */
 class CORE_EXPORT QgsNmeaConnection: public QgsGpsConnection
 {
@@ -32,9 +31,11 @@ class CORE_EXPORT QgsNmeaConnection: public QgsGpsConnection
   public:
 
     /**
-     * \brief Constructs a QgsNmeaConnection with given \a device.
+     * Constructs a QgsNmeaConnection with given \a device.
+     *
+     * Ownership of \a device is transferred to the connection.
      */
-    QgsNmeaConnection( QIODevice *device );
+    QgsNmeaConnection( QIODevice *device SIP_TRANSFER );
 
   protected slots:
     //! Parse available data source content
@@ -60,6 +61,10 @@ class CORE_EXPORT QgsNmeaConnection: public QgsGpsConnection
     void processGstSentence( const char *data, int len );
     //! process HDT sentence
     void processHdtSentence( const char *data, int len );
+    //! process HCHDG sentence
+    void processHchdgSentence( const char *data, int len );
+    //! process HCHDT sentence
+    void processHchdtSentence( const char *data, int len );
 };
 
 #endif // QGSNMEACONNECTION_H
