@@ -28,7 +28,6 @@ class QgsLineString;
  * \ingroup core
  * \class QgsPolygon
  * \brief Polygon geometry type.
- * \since QGIS 2.10
  */
 class CORE_EXPORT QgsPolygon: public QgsCurvePolygon
 {
@@ -55,6 +54,7 @@ class CORE_EXPORT QgsPolygon: public QgsCurvePolygon
     bool fromWkb( QgsConstWkbPtr &wkb ) override;
     int wkbSize( QgsAbstractGeometry::WkbFlags flags = QgsAbstractGeometry::WkbFlags() ) const override;
     QByteArray asWkb( QgsAbstractGeometry::WkbFlags flags = QgsAbstractGeometry::WkbFlags() ) const override;
+    QString asWkt( int precision = 17 ) const override;
     QgsPolygon *surfaceToPolygon() const override SIP_FACTORY;
 
     /**
@@ -73,7 +73,6 @@ class CORE_EXPORT QgsPolygon: public QgsCurvePolygon
      * Returns the distance from a point to the boundary of the polygon (either the
      * exterior ring or any closer interior rings). The returned distance will be
      * negative if the point lies outside the polygon.
-     * \since QGIS 3.0
      */
     double pointDistanceToBoundary( double x, double y ) const;
 
@@ -84,17 +83,16 @@ class CORE_EXPORT QgsPolygon: public QgsCurvePolygon
      * Should be used by qgsgeometry_cast<QgsPolygon *>( geometry ).
      *
      * \note Not available in Python. Objects will be automatically be converted to the appropriate target type.
-     * \since QGIS 3.0
      */
     inline static const QgsPolygon *cast( const QgsAbstractGeometry *geom )
     {
       if ( !geom )
         return nullptr;
 
-      QgsWkbTypes::Type flatType = QgsWkbTypes::flatType( geom->wkbType() );
+      const Qgis::WkbType flatType = QgsWkbTypes::flatType( geom->wkbType() );
 
-      if ( flatType == QgsWkbTypes::Polygon
-           || flatType == QgsWkbTypes::Triangle )
+      if ( flatType == Qgis::WkbType::Polygon
+           || flatType == Qgis::WkbType::Triangle )
         return static_cast<const QgsPolygon *>( geom );
       return nullptr;
     }
