@@ -32,6 +32,7 @@ class QgsMapClippingRegion;
 
 #include <QList>
 #include <QPainter>
+#include <QElapsedTimer>
 
 typedef QList<int> QgsAttributeList;
 
@@ -49,28 +50,9 @@ class QgsVectorLayerDiagramProvider;
 
 /**
  * \ingroup core
- * Interruption checker used by QgsVectorLayerRenderer::render()
- * \note not available in Python bindings
- */
-class QgsVectorLayerRendererInterruptionChecker: public QgsFeedback
-{
-    Q_OBJECT
-
-  public:
-    //! Constructor
-    explicit QgsVectorLayerRendererInterruptionChecker( const QgsRenderContext &context );
-
-  private:
-    const QgsRenderContext &mContext;
-    QTimer *mTimer = nullptr;
-};
-
-/**
- * \ingroup core
- * Implementation of threaded rendering for vector layers.
+ * \brief Implementation of threaded rendering for vector layers.
  *
  * \note not available in Python bindings
- * \since QGIS 2.4
  */
 class QgsVectorLayerRenderer : public QgsMapLayerRenderer
 {
@@ -78,6 +60,7 @@ class QgsVectorLayerRenderer : public QgsMapLayerRenderer
     QgsVectorLayerRenderer( QgsVectorLayer *layer, QgsRenderContext &context );
     ~QgsVectorLayerRenderer() override;
     QgsFeedback *feedback() const override;
+    bool forceRasterRender() const override;
 
     /**
      * Returns the feature renderer.
@@ -88,6 +71,8 @@ class QgsVectorLayerRenderer : public QgsMapLayerRenderer
 
     bool render() override;
 
+    void setLayerRenderingTimeHint( int time ) override;
+
   private:
 
     /**
@@ -99,25 +84,28 @@ class QgsVectorLayerRenderer : public QgsMapLayerRenderer
     void prepareDiagrams( QgsVectorLayer *layer, QSet<QString> &attributeNames );
 
     /**
-     * Draw layer with renderer V2. QgsFeatureRenderer::startRender() needs to be called before using this method
+     * Draw layer with \a renderer. QgsFeatureRenderer::startRender() needs to be called before using this method
      */
-    void drawRenderer( QgsFeatureIterator &fit );
+    void drawRenderer( QgsFeatureRenderer *renderer, QgsFeatureIterator &fit );
 
     /**
-     * Draw layer with renderer V2 using symbol levels. QgsFeatureRenderer::startRender() needs to be called before using this method
+     * Draw layer with \a renderer using symbol levels. QgsFeatureRenderer::startRender() needs to be called before using this method
      */
-    void drawRendererLevels( QgsFeatureIterator &fit );
+    void drawRendererLevels( QgsFeatureRenderer *renderer, QgsFeatureIterator &fit );
 
     //! Stop version 2 renderer and selected renderer (if required)
-    void stopRenderer( QgsSingleSymbolRenderer *selRenderer );
+    void stopRenderer( QgsFeatureRenderer *renderer, QgsSingleSymbolRenderer *selRenderer );
 
 
-  protected:
+    bool renderInternal( QgsFeatureRenderer *renderer, int rendererIndex );
 
-    std::unique_ptr< QgsVectorLayerRendererInterruptionChecker > mInterruptionChecker;
+  private:
+
+    std::unique_ptr<QgsFeedback> mFeedback = nullptr;
 
     //! The rendered layer
     QgsVectorLayer *mLayer = nullptr;
+    QString mLayerName;
 
     QgsFields mFields; // TODO: use fields from mSource
 
@@ -125,24 +113,20 @@ class QgsVectorLayerRenderer : public QgsMapLayerRenderer
 
     QString mTemporalFilter;
 
-    QgsVectorLayerFeatureSource *mSource = nullptr;
+    std::unique_ptr< QgsVectorLayerFeatureSource > mSource;
 
     QgsFeatureRenderer *mRenderer = nullptr;
+    std::vector< std::unique_ptr< QgsFeatureRenderer> > mRenderers;
 
     bool mDrawVertexMarkers;
     bool mVertexMarkerOnlyForSelection;
-    int mVertexMarkerStyle = 0;
+    Qgis::VertexMarkerType mVertexMarkerStyle = Qgis::VertexMarkerType::SemiTransparentCircle;
     double mVertexMarkerSize = 2.0;
 
-    QgsWkbTypes::GeometryType mGeometryType;
+    Qgis::GeometryType mGeometryType;
 
     QSet<QString> mAttrNames;
 
-    //! used with old labeling engine (QgsPalLabeling): whether labeling is enabled
-    bool mLabeling;
-    //! used with new labeling engine (QgsPalLabeling): whether diagrams are enabled
-    bool mDiagrams;
-
     /**
      * used with new labeling engine (QgsLabelingEngine): provider for labels.
      * may be NULLPTR. no need to delete: if exists it is owned by labeling engine
@@ -167,6 +151,18 @@ class QgsVectorLayerRenderer : public QgsMapLayerRenderer
     bool mApplyClipGeometries = false;
     QgsGeometry mLabelClipFeatureGeom;
     bool mApplyLabelClipGeometries = false;
+    bool mForceRasterRender = false;
+
+    int mRenderTimeHint = 0;
+    bool mBlockRenderUpdates = false;
+    QElapsedTimer mElapsedTimer;
+
+    bool mNoSetLayerExpressionContext = false;
+
+    bool mEnableProfile = false;
+    quint64 mPreparationTime = 0;
+
+    std::unique_ptr< QgsSymbol > mSelectionSymbol;
 
 };
 
