@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     ProcessingConfig.py
@@ -22,6 +20,7 @@ __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
 import os
+import tempfile
 
 from qgis.PyQt.QtCore import QCoreApplication, QObject, pyqtSignal
 from qgis.core import (NULL,
@@ -43,14 +42,13 @@ settingsWatcher = SettingsWatcher()
 
 
 class ProcessingConfig:
-
     OUTPUT_FOLDER = 'OUTPUTS_FOLDER'
     RASTER_STYLE = 'RASTER_STYLE'
     VECTOR_POINT_STYLE = 'VECTOR_POINT_STYLE'
     VECTOR_LINE_STYLE = 'VECTOR_LINE_STYLE'
     VECTOR_POLYGON_STYLE = 'VECTOR_POLYGON_STYLE'
     FILTER_INVALID_GEOMETRIES = 'FILTER_INVALID_GEOMETRIES'
-    PREFER_FILENAME_AS_LAYER_NAME = 'PREFER_FILENAME_AS_LAYER_NAME'
+    PREFER_FILENAME_AS_LAYER_NAME = 'prefer-filename-as-layer-name'
     KEEP_DIALOG_OPEN = 'KEEP_DIALOG_OPEN'
     PRE_EXECUTION_SCRIPT = 'PRE_EXECUTION_SCRIPT'
     POST_EXECUTION_SCRIPT = 'POST_EXECUTION_SCRIPT'
@@ -59,9 +57,11 @@ class ProcessingConfig:
     SHOW_PROVIDERS_TOOLTIP = 'SHOW_PROVIDERS_TOOLTIP'
     SHOW_ALGORITHMS_KNOWN_ISSUES = 'SHOW_ALGORITHMS_KNOWN_ISSUES'
     MAX_THREADS = 'MAX_THREADS'
-    DEFAULT_OUTPUT_RASTER_LAYER_EXT = 'DefaultOutputRasterLayerExt'
-    DEFAULT_OUTPUT_VECTOR_LAYER_EXT = 'DefaultOutputVectorLayerExt'
-    TEMP_PATH = 'TEMP_PATH'
+    DEFAULT_OUTPUT_RASTER_LAYER_EXT = 'default-output-raster-ext'
+    DEFAULT_OUTPUT_VECTOR_LAYER_EXT = 'default-output-vector-ext'
+    TEMP_PATH = 'temp-path'
+    RESULTS_GROUP_NAME = 'RESULTS_GROUP_NAME'
+    VECTOR_FEATURE_COUNT = 'VECTOR_FEATURE_COUNT'
 
     settings = {}
     settingIcons = {}
@@ -77,7 +77,8 @@ class ProcessingConfig:
         ProcessingConfig.addSetting(Setting(
             ProcessingConfig.tr('General'),
             ProcessingConfig.PREFER_FILENAME_AS_LAYER_NAME,
-            ProcessingConfig.tr('Prefer output filename for layer names'), True))
+            ProcessingConfig.tr('Prefer output filename for layer names'), True,
+            hasSettingEntry=True))
         ProcessingConfig.addSetting(Setting(
             ProcessingConfig.tr('General'),
             ProcessingConfig.SHOW_PROVIDERS_TOOLTIP,
@@ -141,8 +142,8 @@ class ProcessingConfig:
             valuetype=Setting.SELECTION,
             options=invalidFeaturesOptions))
 
-        threads = QgsApplication.maxThreads() # if user specified limit for rendering, lets keep that as default here, otherwise max
-        threads = cpu_count() if threads == -1 else threads # if unset, maxThreads() returns -1
+        threads = QgsApplication.maxThreads()  # if user specified limit for rendering, lets keep that as default here, otherwise max
+        threads = cpu_count() if threads == -1 else threads  # if unset, maxThreads() returns -1
         ProcessingConfig.addSetting(Setting(
             ProcessingConfig.tr('General'),
             ProcessingConfig.MAX_THREADS,
@@ -154,9 +155,10 @@ class ProcessingConfig:
             ProcessingConfig.tr('General'),
             ProcessingConfig.DEFAULT_OUTPUT_VECTOR_LAYER_EXT,
             ProcessingConfig.tr('Default output vector layer extension'),
-            'gpkg',
-            valuetype=Setting.SELECTION,
-            options=extensions))
+            QgsVectorFileWriter.supportedFormatExtensions()[0],
+            valuetype=Setting.SELECTION_STORE_STRING,
+            options=extensions,
+            hasSettingEntry=True))
 
         extensions = QgsRasterFileWriter.supportedFormatExtensions()
         ProcessingConfig.addSetting(Setting(
@@ -164,14 +166,31 @@ class ProcessingConfig:
             ProcessingConfig.DEFAULT_OUTPUT_RASTER_LAYER_EXT,
             ProcessingConfig.tr('Default output raster layer extension'),
             'tif',
-            valuetype=Setting.SELECTION,
-            options=extensions))
+            valuetype=Setting.SELECTION_STORE_STRING,
+            options=extensions,
+            hasSettingEntry=True))
 
         ProcessingConfig.addSetting(Setting(
             ProcessingConfig.tr('General'),
             ProcessingConfig.TEMP_PATH,
-            ProcessingConfig.tr('Temporary output folder path'), QgsProcessingUtils.tempFolder(),
-            valuetype=Setting.FOLDER))
+            ProcessingConfig.tr('Override temporary output folder path'), None,
+            valuetype=Setting.FOLDER,
+            placeholder=ProcessingConfig.tr('Leave blank for default'),
+            hasSettingEntry=True))
+
+        ProcessingConfig.addSetting(Setting(
+            ProcessingConfig.tr('General'),
+            ProcessingConfig.RESULTS_GROUP_NAME,
+            ProcessingConfig.tr("Results group name"),
+            "",
+            valuetype=Setting.STRING,
+            placeholder=ProcessingConfig.tr("Leave blank to avoid loading results in a predetermined group")
+        ))
+
+        ProcessingConfig.addSetting(Setting(
+            ProcessingConfig.tr('General'),
+            ProcessingConfig.VECTOR_FEATURE_COUNT,
+            ProcessingConfig.tr('Show feature count for output vector layers'), False))
 
     @staticmethod
     def setGroupIcon(group, icon):
@@ -214,21 +233,19 @@ class ProcessingConfig:
 
     @staticmethod
     def getSetting(name, readable=False):
-        if name in list(ProcessingConfig.settings.keys()):
-            v = ProcessingConfig.settings[name].value
-            try:
-                if v == NULL:
-                    v = None
-            except:
-                pass
-            if ProcessingConfig.settings[name].valuetype == Setting.SELECTION:
-                if readable:
-                    return v
-                return ProcessingConfig.settings[name].options.index(v)
-            else:
-                return v
-        else:
+        if name not in list(ProcessingConfig.settings.keys()):
             return None
+        v = ProcessingConfig.settings[name].value
+        try:
+            if v == NULL:
+                v = None
+        except:
+            pass
+        if ProcessingConfig.settings[name].valuetype != Setting.SELECTION:
+            return v
+        if readable:
+            return v
+        return ProcessingConfig.settings[name].options.index(v)
 
     @staticmethod
     def setSettingValue(name, value):
@@ -247,7 +264,6 @@ class ProcessingConfig:
 
 
 class Setting:
-
     """A simple config parameter that will appear on the config dialog.
     """
     STRING = 0
@@ -257,17 +273,22 @@ class Setting:
     FLOAT = 4
     INT = 5
     MULTIPLE_FOLDERS = 6
+    SELECTION_STORE_STRING = 7
 
     def __init__(self, group, name, description, default, hidden=False, valuetype=None,
-                 validator=None, options=None):
+                 validator=None, options=None, placeholder="", hasSettingEntry=False):
+        """
+        hasSettingEntry is true if the given setting is part of QgsSettingsRegistry entries
+        """
         self.group = group
         self.name = name
-        self.qname = "Processing/Configuration/" + self.name
+        self.qname = ("qgis/configuration/" if hasSettingEntry else "Processing/Configuration/") + self.name
         self.description = description
         self.default = default
         self.hidden = hidden
         self.valuetype = valuetype
         self.options = options
+        self.placeholder = placeholder
 
         if self.valuetype is None:
             if isinstance(default, int):
@@ -282,6 +303,7 @@ class Setting:
                         float(v)
                     except ValueError:
                         raise ValueError(self.tr('Wrong parameter value:\n{0}').format(v))
+
                 validator = checkFloat
             elif self.valuetype == self.INT:
                 def checkInt(v):
@@ -289,11 +311,13 @@ class Setting:
                         int(v)
                     except ValueError:
                         raise ValueError(self.tr('Wrong parameter value:\n{0}').format(v))
+
                 validator = checkInt
             elif self.valuetype in [self.FILE, self.FOLDER]:
                 def checkFileOrFolder(v):
                     if v and not os.path.exists(v):
                         raise ValueError(self.tr('Specified path does not exist:\n{0}').format(v))
+
                 validator = checkFileOrFolder
             elif self.valuetype == self.MULTIPLE_FOLDERS:
                 def checkMultipleFolders(v):
@@ -301,6 +325,7 @@ class Setting:
                     for f in folders:
                         if f and not os.path.exists(f):
                             raise ValueError(self.tr('Specified path does not exist:\n{0}').format(f))
+
                 validator = checkMultipleFolders
             else:
                 def validator(x):
@@ -331,6 +356,9 @@ class Setting:
     def save(self, qsettings=None):
         if not qsettings:
             qsettings = QgsSettings()
+        if self.value == self.default:
+            qsettings.remove(self.qname)
+            return
         if self.valuetype == self.SELECTION:
             qsettings.setValue(self.qname, self.options.index(self.value))
         else:
